<?php

namespace Lc\SovBundle\Form\Type\Crud;

use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\EntityRepository;
use EasyCorp\Bundle\EasyAdminBundle\Config\Crud;
use EasyCorp\Bundle\EasyAdminBundle\Contracts\Field\FieldInterface;
use EasyCorp\Bundle\EasyAdminBundle\Dto\FieldDto;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\DateTimeType;
use Symfony\Component\Form\Extension\Core\Type\DateType;
use Symfony\Component\Form\Extension\Core\Type\FormType;
use Symfony\Component\Form\Extension\Core\Type\IntegerType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Contracts\Translation\TranslatorInterface;


class IndexFilterType extends AbstractType
{
    protected $em;
    protected $translator;
    protected $utils;

    public function __construct(EntityManagerInterface $entityManager, TranslatorInterface $translator)
    {
        $this->em = $entityManager;
        $this->translator = $translator;
    }


    public function buildForm(FormBuilderInterface $builder, array $options)
    {


        foreach ($options['fields'] as $field) {
            if($field instanceof FieldInterface) {
                $fieldDto = $field->getAsDto();
            }else{
                $fieldDto = $field;
            }
            if ($fieldDto->isDisplayedOn(Crud::PAGE_INDEX)) {

                switch ($fieldDto->getFormType()) {
                    case 'option':

                        $refl = new \ReflectionClass($options['entity_class']);

                        $choiceOptions = array();
                        foreach ($refl->getConstants() as $key => $value) {
                            if (stripos($key, $this->utils->snakeCase($field['property'])) !== false) {
                                $choiceOptions[$this->translator->trans(
                                        'field.'.$options['entity_name'].'.'.$field['property'].'Options.'.$value
                                )] = $value;
                            }
                        }

                        $builder->add(
                                $field['property'],
                                ChoiceType::class,
                                array(
                                        'required' => false,
                                        'choices' => $choiceOptions,
                                        'attr' => array(
                                                'class' => 'select2 input-sm',
                                                'form' => 'filters-form',
                                        ),
                                )
                        );
                        break;
                    case 'integer':
                        $builder->add(
                                $field['property'],
                                TextType::class,
                                array(
                                        'required' => false,
                                        'attr' => array(
                                                'class' => ' input-sm',
                                                'form' => 'filters-form',
                                        ),
                                )
                        );
                        break;
                    case TextType::class:
                        $builder->add(
                                str_replace('.', '_', $fieldDto->getProperty()),
                                TextType::class,
                                array(
                                        'required' => false,
                                        'attr' => array(
                                                'class' => ' input-sm',
                                                'form' => 'filters-form',
                                        ),
                                )
                        );
                        break;
                    case 'toggle':
                        $builder->add(
                                $field['property'],
                                ChoiceType::class,
                                array(
                                        'choices' => array(
                                                'Oui' => 1,
                                                'Non' => 0,
                                        ),
                                        'placeholder' => '--',
                                        'required' => false,
                                        'attr' => array(
                                                'class' => 'select2 input-sm',
                                                'form' => 'filters-form',
                                        ),
                                )
                        );
                        break;
                    case 'datetime':
                    case 'date':
                        $builder->add(
                                $builder->create($field['property'], FormType::class, array('inherit_data' => true))
                                        ->add(
                                                'dateStart',
                                                DateType::class,
                                                array(
                                                        'widget' => 'single_text',
                                                        'required' => false,
                                                )
                                        )
                                        ->add(
                                                'dateEnd',
                                                DateType::class,
                                                array(
                                                        'widget' => 'single_text',
                                                        'required' => false,

                                                )
                                        )
                        );
                        break;
                    case 'association':
                        $classImplements = class_implements($field['targetEntity']);
                        $builder->add(
                                $field['property'],
                                EntityType::class,
                                array(
                                        'class' => $field['targetEntity'],
                                        'placeholder' => '--',
                                        'query_builder' => function (EntityRepository $repo) use ($classImplements) {
                                            return $repo->createQueryBuilder('entity');
                                        },
                                        'required' => false,
                                        'attr' => array(
                                                'class' => 'select2 input-sm',
                                                'form' => 'filters-form',
                                        ),


                                )
                        );
                        break;
                    case 'dateinterval':

                        break;
                    case 'float':

                        break;
                }
            }
        }
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults(
                [
                        'label' => false,
                        'csrf_protection' => false,
                        //'translation_domain' => 'lcshop',
                        'fields' => false,
                        'entity_name' => false,
                        'entity_class' => false,
                    //'entityClass' => false
                ]
        );
    }
}
